package ch.std.jumpstartwebfluxfn.rest;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.reactive.function.server.ServerResponse;

import ch.std.jumpstartwebfluxfn.beans.ReactiveCitiesBean;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

public class CityHandler {
	Logger logger = LoggerFactory.getLogger(CityHandler.class);

	public Mono<ServerResponse> getAllCityNames(ServerRequest request) {
		logger.info("WebfluxfnCityHandler.getAllCityNames");
		Flux<String> fluxCityNames =  ReactiveCitiesBean.getCities();
		return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(fluxCityNames, String.class);
	}

	public Mono<ServerResponse> getAllCityMap(ServerRequest request) {
		logger.info("WebfluxfnCityHandler.getAllCityMap");
		Flux<Map> fluxCityMap = ReactiveCitiesBean.getCities().map(city -> {
				Map<String, Object> map = new HashMap<>();
				map.put("city", city);
				return map;
		});		
		return ServerResponse.ok().contentType(MediaType.APPLICATION_STREAM_JSON).body(fluxCityMap, Map.class);
	}
	
	public Mono<ServerResponse> getCity(ServerRequest request) {
		Object value = request.queryParam("value").orElse("");
		logger.info("WebfluxfnCityHandler.getCity value = " + value);
		Mono<String> monoCity = ReactiveCitiesBean.find(value.toString());
		return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(monoCity, String.class);
	}
		
	public Mono<ServerResponse> getCityMap(ServerRequest request) {
		Object value = request.queryParam("value").orElse("");
		logger.info("WebfluxfnCityHandler.getCityMap value = " + value);
		Mono<Map> monoCity = ReactiveCitiesBean.find(value.toString()).map(city -> Collections.singletonMap("city", city));
		return ServerResponse.ok().contentType(MediaType.APPLICATION_STREAM_JSON).body(monoCity, Map.class);
	}
	
}
